@@ -7,7 +7,8 @@ from gevent.socket import socket
 from slimta.smtp.io import IO
 from slimta.smtp.auth import Auth, CredentialsInvalidError, ServerAuthError, \
                              InvalidMechanismError, AuthenticationCanceled
-from slimta.smtp.auth.mechanisms import *
+from slimta.smtp.auth.standard import *
+from slimta.smtp.auth.oauth import OAuth2
 
 
 class StaticCramMd5(CramMd5):
@@ -31,10 +32,19 @@ class FakeAuth(Auth):
             raise CredentialsInvalidError()
         return 'testpassword', None
 
-    def get_available_mechanisms(self, encrypted=False):
+    def get_available_mechanisms(self, encrypted):
         return [Plain, Login, StaticCramMd5]
 
 
+class FakeAuthNoSecure(Auth):
+
+    def get_available_mechanisms(self, encrypted):
+        if encrypted:
+            return [Plain, Login]
+        else:
+            return []
+
+
 class FakeSession(object):
 
     def __init__(self, encrypted):
@@ -60,6 +70,13 @@ class TestSmtpAuth(MoxTestBase):
         auth = Auth(FakeSession(True))
         self.assertEqual('CRAM-MD5 PLAIN LOGIN', str(auth))
 
+    def test_str_no_secure_mechanisms(self):
+        auth = FakeAuthNoSecure(FakeSession(True))
+        self.assertEqual('PLAIN LOGIN', str(auth))
+        auth = FakeAuthNoSecure(FakeSession(False))
+        with self.assertRaises(ValueError):
+            str(auth)
+
     def test_unimplemented_means_invalid(self):
         auth = Auth(None)
         with self.assertRaises(CredentialsInvalidError):
@@ -214,5 +231,25 @@ class TestSmtpAuth(MoxTestBase):
         self.assertEqual('535', reply.code)
         self.assertEqual('5.0.0 Nope!', reply.message)
 
+    def test_client_xoauth2(self):
+        self.sock.sendall('AUTH XOAUTH2 dXNlcj10ZXN0QGV4YW1wbGUuY29tAWF1dGg9QmVhcmVyYXNkZgEB\r\n')
+        self.sock.recv(IsA(int)).AndReturn('235 Ok\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = OAuth2.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('235', reply.code)
+        self.assertEqual('2.0.0 Ok', reply.message)
+
+    def test_client_xoauth2_error(self):
+        self.sock.sendall('AUTH XOAUTH2 dXNlcj10ZXN0QGV4YW1wbGUuY29tAWF1dGg9QmVhcmVyYXNkZgEB\r\n')
+        self.sock.recv(IsA(int)).AndReturn('334 eyJzdGF0dXMiOiI0MDEiLCJzY2hlbWVzIjoiYmVhcmVyIG1hYyIsInNjb3BlIjoiaHR0cHM6Ly9tYWlsLmdvb2dsZS5jb20vIn0K\r\n')
+        self.sock.sendall('\r\n')
+        self.sock.recv(IsA(int)).AndReturn('535 Nope!\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = OAuth2.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('535', reply.code)
+        self.assertEqual('5.0.0 Nope!', reply.message)
+
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
